#############################################
# Simulation file for 'Sequential Selection #
# Procedures and FDR Control' (M. Grazier   #
# G'Sell et al.                             #
#############################################
###################
# Mattia Ciollaro #
###################

source( '~/Desktop/36-825 Statistics Journal Club/My Presentation/Notes/simulation.functions.R' )

#############################################
# A simple example with ordererd hypotheses #
#############################################
# number of hypotheses
m <- 100

# number of false null hypotheses
s <- 20

# number of simulations
n.simulations <- 500

# alpha
alpha <- c( 0.05, 0.10, 0.15, 0.20, 0.25 )

# parameter for the beta distribution
# which generates the non-uniform p values
# for the false null hypotheses
beta.easy <- 23
beta.medium <- 14
beta.hard <- 8
beta.Hard <- 4

# plot the three beta distribution and the
# standard uniform for reference
t <- seq( 0, 1, by=0.01 )
y.easy <- dbeta( t, 1, beta.easy )
y.medium <- dbeta( t, 1, beta.medium )
y.hard <- dbeta( t, 1, beta.hard )
y.Hard <- dbeta( t, 1, beta.Hard )

my.lwd=1

my.ylim <- c( 0, max( y.easy ) )
    
plot( t, y.hard, type='l', lwd=3*my.lwd, xlab='', ylab='', ylim=my.ylim, main='Beta distributions for p-values simulation' )
lines( t, y.medium, lwd=2*my.lwd )
lines( t, y.easy, lwd=my.lwd )
lines( t, y.Hard, lwd=4*my.lwd )
lines( t, rep( 1, length( t ) ), col='red', lwd=my.lwd )
abline( h=0 )

# simulation - easy scenario
my.easy.simulation.ord <- whole.simulation( n.simulations=n.simulations, m=m, s=s, beta=beta.easy, alpha=alpha, r=r, probs.function=probs.function, separated=TRUE )

# simulation - medium scenario
my.medium.simulation.ord <- whole.simulation( n.simulations=n.simulations, m=m, s=s, beta=beta.medium, alpha=alpha, r=r, probs.function=probs.function, separated=TRUE )

# simulation - hard scenario
my.hard.simulation.ord <- whole.simulation( n.simulations=n.simulations, m=m, s=s, beta=beta.hard, alpha=alpha, r=r, probs.function=probs.function, separated=TRUE )

# simulation - very hard scenario
my.Hard.simulation.ord <- whole.simulation( n.simulations=n.simulations, m=m, s=s, beta=beta.Hard, alpha=alpha, r=r, probs.function=probs.function, separated=TRUE )

# PLOTS
#
# plot p-values
dev.new( )
par( mfrow=c(2,2) )

only.first <- 50

my.cex <- 0.5

plot.pvalues( my.easy.simulation.ord$pvalues, my.easy.simulation.ord$false.null.indices, main='Easy', only.first=only.first, cex=my.cex )
plot.pvalues( my.medium.simulation.ord$pvalues, my.medium.simulation.ord$false.null.indices, main='Medium', only.first=only.first, cex=my.cex )
plot.pvalues( my.hard.simulation.ord$pvalues, my.hard.simulation.ord$false.null.indices, main='Hard', only.first=only.first, cex=my.cex )
plot.pvalues( my.Hard.simulation.ord$pvalues, my.Hard.simulation.ord$false.null.indices, main='Very Hard', only.first=only.first, cex=my.cex )

# plot fdr and power analysis
plot.simulation.results( my.easy.simulation.ord, main='Easy' )
plot.simulation.results( my.medium.simulation.ord, main='Medium' )
plot.simulation.results( my.hard.simulation.ord, main='Hard' )
plot.simulation.results( my.Hard.simulation.ord, main='Very Hard' )


###############################################
# A simple example with unordererd hypotheses #
###############################################
# number of hypotheses
m <- 100

# number of false null hypotheses
s <- 20

# mixing of true and false null hypotheses
# try r=0.8 (roughly 2-3 false nulls after the 20th)
# and r=0.95 (roughy 4-5 nulls after the 20th)
r <- 0.8

# number of simulations
n.simulations <- 500

# alpha
alpha <- c( 0.05, 0.10, 0.15, 0.20, 0.25 )

# probs function
probs.function <- 'mine'

# parameter for the beta distribution
# which generates the non-uniform p values
# for the false null hypotheses
beta.easy <- 23
beta.medium <- 14
beta.hard <- 8
beta.Hard <- 4

# plot the three beta distribution and the
# standard uniform for reference
t <- seq( 0, 1, by=0.01 )
y.easy <- dbeta( t, 1, beta.easy )
y.medium <- dbeta( t, 1, beta.medium )
y.hard <- dbeta( t, 1, beta.hard )
y.Hard <- dbeta( t, 1, beta.Hard )

my.lwd=2

dev.new()
par( mfrow=c( 1,2 ) )
plot( t, y.hard, type='l', lwd=3*my.lwd, xlab='', ylab='', ylim=my.ylim, main='Beta distributions for p-values simulation' )
lines( t, y.medium, lwd=2*my.lwd )
lines( t, y.easy, lwd=my.lwd )
lines( t, y.Hard, lwd=4*my.lwd )
lines( t, rep( 1, length( t ) ), col='red' )
abline( h=0 )

if( probs.function == 'mine' ){
    
    barplot( probs( r=r ), main='Distribution for index sampling' )
    abline( v=s )

}
if( probs.function == 'Max' ){

    bbeta <- readline('enter beta ')
    bbeta <- as.numeric( bbeta ) 

    barplot( probs.Max( m, gamma=bbeta ), main='Distribution for index sampling' )
    abline( v=s )
    
}

# simulation - easy scenario
my.easy.simulation <- whole.simulation( n.simulations=n.simulations, m=m, s=s, beta=beta.easy, alpha=alpha, r=r, probs.function=probs.function )

# simulation - medium scenario
my.medium.simulation <- whole.simulation( n.simulations=n.simulations, m=m, s=s, beta=beta.medium, alpha=alpha, r=r, probs.function=probs.function )

# simulation - hard scenario
my.hard.simulation <- whole.simulation( n.simulations=n.simulations, m=m, s=s, beta=beta.hard, alpha=alpha, r=r, probs.function=probs.function )

# simulation - very hard scenario
my.Hard.simulation <- whole.simulation( n.simulations=n.simulations, m=m, s=s, beta=beta.Hard, alpha=alpha, r=r, probs.function=probs.function)

# PLOTS
#
# plot p-values
dev.new()
par( mfrow=c(2,2) )

only.first <- 50

my.cex <- 0.4

plot.pvalues( my.easy.simulation$pvalues, my.easy.simulation$false.null.indices, main='Easy', only.first=only.first, cex=my.cex )
plot.pvalues( my.medium.simulation$pvalues, my.medium.simulation$false.null.indices, main='Medium', only.first=only.first, cex=my.cex )
plot.pvalues( my.hard.simulation$pvalues, my.hard.simulation$false.null.indices, main='Hard', only.first=only.first, cex=my.cex )
plot.pvalues( my.Hard.simulation$pvalues, my.Hard.simulation$false.null.indices, main='Very Hard', only.first=only.first, cex=my.cex )

# plot fdr and power analysis
plot.simulation.results( my.easy.simulation, main='Easy' )
plot.simulation.results( my.medium.simulation, main='Medium' )
plot.simulation.results( my.hard.simulation, main='Hard' )
plot.simulation.results( my.Hard.simulation, main='Very Hard' )
